/** \file RTDomain.cpp 
*COPYRIGHT (C) 2006  Michele Basso   (basso@dsi.unifi.it)
*                    Marco Romagnoli (romagnoli@control.dsi.unifi.it)
*                    Massimo Vassalli(vassalli@inoa.it)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include "RTDomain.h"

namespace rtaixml_domain
{
	RTDomain::~RTDomain()
	{
	}
	
	void RTDomain::Initialize()
	{
		if(IS_DEBUG) std::cout<<"|| RTDomain::Initialize() ||"<<std::endl;
		//At this time security policy are taken into account in the sw architecture, but are not
		// implemented; there is only a fake class.
		cpSecure = new CheckPointFake();
		
		//Inizialization of the random number generator.
		srand((unsigned)time(0)); 
		if(IS_DEBUG) { std::cout<<"seed:"<<(unsigned)time(0)<<std::endl; std::fflush(stdout);}

		//... perform necessary DEFAULT instance initializations 
		iBasePortXML 	= DEFAULT_BASE_PORT;
		iMaxPortXML 	= DEFAULT_MAX_PORT;
		sMasterName 	= DEFAULT_MASTER_NAME;
		sRtaiNames 		= DEFAULT_RTAI_NAMES; 
		
		//Load the configuration file
		LoadConfigFile();
		
		if(IS_DEBUG) std::cout<<"RTDomain::Initialize(): XML Port interval "<< iBasePortXML <<" - "<< iMaxPortXML <<"."<<std::endl;
		if(IS_DEBUG) std::cout<<"RTDomain::Initialize(): sMasterName "<<sMasterName<<std::endl;
		if(IS_DEBUG) std::cout<<"|| End of RTDomain::Initialize() ||"<<std::endl;
	}
	
	std::string RTDomain::get_random_string(unsigned int lenght)
	{
		std::string random_string(lenght,'x');
		int random_integer;
		
		//pause to improve the random generation
		clock_t start_time = clock();
		while((clock() - start_time) < 0.1 * CLOCKS_PER_SEC) {}
		
		for(unsigned int index=0; index<lenght; index++){
			random_integer = (rand()%36);
			//Numbers
			if(random_integer<10)
				random_string[index]=48+random_integer;
			//Capitol Letter
			else if(random_integer<36)
				random_string[index]=65+random_integer-10;
			else
				random_string[index]=65;
			//Small Letter
			//else if(random_integer<62)
			//	random_string[index]=97+random_integer-36;
			//else
			//	random_string[index]=97;
		}
		return random_string;
	}
	
	
	int RTDomain::getPortXML()
	{
		return iBasePortXML;
	}
	
	int RTDomain::getMaxPortXML()
	{
		return iMaxPortXML;
	}
	
	//Base port is the master. If base port is incompatible with max port
	bool RTDomain::setBasePortXML(int iPort)
	{
		if(iPort>1024 && iPort<65537-2)
		{
			iBasePortXML=iPort;
			if(iMaxPortXML<(iBasePortXML+2))
				iMaxPortXML=iBasePortXML+2;
			return true;
		}
		return false;
	}
	
	bool RTDomain::setMaxPortXML(int iPort)
	{
		if(iPort>1024 && iPort<65537 && iPort>iBasePortXML)
		{
			iMaxPortXML=iPort;
			return true;
		}
		return false;
	}
	
	std::string RTDomain::getMasterName()
	{
		if(IS_DEBUG) { std::cout<<"RTDomain::getMasterName(): Master Name: "<< sMasterName << std::endl;std::fflush(stdout); }
		return sMasterName;
	}
		
	void RTDomain::setMasterName(std::string sMN)	
	{
		sMasterName = sMN;
	}
		
			
	bool RTDomain::IsThereATarget(std::string sTaskName)
	{
		if(IS_DEBUG) std::cout << "|| RTDomain::IsThereATarget ||" << std::endl;
		bool bExist=false;
		std::string sLine;

		std::ifstream myfile (sRtaiNames.c_str());

		if (myfile.is_open())
		{
			while (! myfile.eof() && !bExist)
			{
				getline (myfile,sLine);
				bExist = parse_line(sLine,sTaskName);		
			}
			myfile.close();
		}
		else {
			bExist=false;
		}
		
		if(!bExist)
			RemoveActor(sTaskName);

		if(IS_DEBUG) std::cout << "RTDomain::IsThereATarget - TaskName " << sTaskName << (bExist?" ":" NOT ") << "exists!"<< std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomain::IsThereATarget ||" << std::endl;
		return bExist;
	}
	
	bool RTDomain::parse_line(std::string sLine,std::string sToken)
	{
		StringTokenizer strtok = StringTokenizer(sLine," ");
		int iCnt = strtok.countTokens();
		for(int i = 0; i < iCnt; i++)
		{
			std::string sTemp =strtok.nextToken();
			if(sTemp.compare(sToken)==0) {
				return true;
			}
		}
		return false;	
	}

	bool RTDomain::SecureCheck()
	{
		if(cpSecure->CheckPolicy()==0)
			return true;
		return false;
	}
	
	void RTDomain::LoadConfigFile()
	{
		if(IS_DEBUG) std::cout << "|| RTDomain::LoadConfigFile ||" << std::endl;
		std::string sLine,sVar,sValue;
		int iParam = 0;
		
		std::ifstream myfile (CONFIGURATION_FILE);

		if (myfile.is_open())
		{
			while (! myfile.eof() )
			{
				getline (myfile,sLine);
				if(sLine[0]!=COMMENT_VALUE)
				{
					StringTokenizer strtok = StringTokenizer(sLine," ");
					iParam++;
					int iCnt = strtok.countTokens();
					if(iCnt==2)
					{
						sVar = strtok.nextToken();
						sValue = strtok.nextToken();
						if(sVar.compare("BasePortXML")==0)
							iBasePortXML = atoi(sValue.c_str());
						else if(sVar.compare("MaxPortXML")==0)
							iMaxPortXML = atoi(sValue.c_str());
						else if(sVar.compare("RtaiNames")==0)
							sRtaiNames = sValue;
						else if(sVar.compare("MasterName")==0)
							this->setMasterName(sValue);
/*						else
							if(IS_DEBUG) std::cout << "RTDomain::LoadConfigFile - Check the configuration file for errors!!!" << std::endl;*/
					}
				}
			}
			myfile.close();
		}
		
		if(IS_DEBUG) std::cout << "RTDomain::LoadConfigFile - " << iParam << " parameter(s) set." << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomain::LoadConfigFile ||" << std::endl;
	}

} //end namespace rtaixml_domain
