/** \file rtaixml_master.h
*COPYRIGHT (C) 2006  Michele Basso   (basso@dsi.unifi.it)
*                    Marco Romagnoli (romagnoli@control.dsi.unifi.it)
*                    Massimo Vassalli(vassalli@inoa.it)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

/**
 * \mainpage RTAI-XML server
 * 
 * RTAI-XML is a server component of the Real Time Application Interface (RTAI) 
 * project, implementing a service oriented way to design and develop Real Time 
 * (RT) control applications.
 * This project was born to fulfill the needs of a university group, mainly 
 * focused to have a flexible platform for learning control systems design, 
 * allowing the students to test their programs remotely, over the Internet. 
 * Leaving the first wishfull thinking and going to the real implementation 
 * gave rise to the alpha version of RTAI-XML, that showed the potential impact 
 * of the basic idea of a net separation of Hard and Soft Real Time tasks in the 
 * programmation logic. What was necessary to assure that students could not 
 * crash the RT process, is now becoming a new RTAI paradigm.
 * \section first RTAI-XML in a nutshell
 * RTAI-XML consists of a server component waiting for incoming calls on a 
 * box where a Real Time process, the Target, is running (or ready to). 
 * A generic client program, the Host, can reach the server through the TCP/IP 
 * network, using a standard protocol based on XML, and hence interact with the 
 * Target, in order to monitor the status of the RT process, to see the signals 
 * collected (or generated) by the system and also to fetch and change the RT 
 * parameters (for example, the gains of a PID feedback ring). In other words, 
 * RTAI-XML provides a simple way towards remoting of control applications, 
 * adding flexibility to the RTAI project, without loosing the key features of 
 * an open and standard implementation.
 * The RTAI-XML section of this site presents the details of the implementation. 
 * The general architecture is revised, in order to focus the three key components, 
 * the Server, the Server-Target interface and the Server-Host communication. 
 * The Applications section contains some examples of control systems based on 
 * RTAI-XML and the References section contains hints and links for further 
 * readings on this topic.
 * 
 * For more info about the structure of the framework see page \ref master_slave.
 * 
 * For more info about the differences in connection between old servers (<0.6) 
 * and new servers (>=0.6) see page \ref connect_difference
 *  
 * \section second Getting started
 * Firstly take a look to the RTAI-XML site http://artist.dsi.unifi.it/rtaixml/ and then 
 * download the server component. In addition to this, you can watch at the 
 * applications for having suggestions and coding hints.
 * 
 * 
 * \section third Application: ARTIST
 * 
 * ARTIST (A Real Time Interactive Simulink/Scicos Telelab)
 * is the remote control laboratory at Dipartimento di Sistemi ed Informatica. Below you can 
 * see a snapshot and at http://artist.dsi.unifi.it you can test our laboratory.
 * 
 *  \image html artist.png
 * 
 * \author Marco Romagnoli
 * \author Michele Basso
 * \author Massimo Vassalli
 * \date January 2007
 * \version 0.8.2
 */

/*! \page master_slave Master/Slave Structure
  The master/slave approach is a new feature of the new version of RTAI-XML. The following 
  sections will illustrate in details how this paradigm works. 
  \section sec_1 The structure
  \subsection sec_1_1 Master/Slave
  The framework is composed by many actors: 
  <ul>
  	<li><b>rtmanager</b>: (master) that is the process that manage the real time domain. 
  		There is only one rtmanager. Rtmanager receives the connection requests and starts the rtaixml
  		server suitable to deal with the requested process.
  	<li><b>rtaixml</b>: (slave) there are many rtaixml servers depending on the number of real time task in execution in the 
  		real time domain. Each real time task has its rtaixml server that provides the target information 
  		outside the real time domain. The rtaixml servers are created by rtmanager for every open session and
  		they terminate automously when the target died.
  	<li><b>targets</b>: they are the real time processes created by users.
  	<li><b>clients</b>: they are external client (i.e. jRtaiLab) that can be connected to the domain.
  </ul>
  The following Figure shows the functional structure of the domain:
  
  \image html functional.jpg
  
  \subsection sec_1_2 Standalone
  The framework could also operate in standalone mode, where the actors are reduced to:
  <ul>
  	<li><b>rtaixml</b>: (standalone) There is only one rtaixml server that can manage only one target at time. 
  		This server receive the connection request that in the other mode are managed by rtmanager.
  		<b>IMPORTANT</b>: the interface to request a connection to rtmanager or rtaixml (in standalone mode) is 
  		the same, so the client remain exactly the same. 
  	<li><b>targets</b>: they are the real time processes created by users, but only one at time can be connected
  		with the rtaixml server and consequentially with outside the real time domain.
  	<li><b>client</b>: Only one client at time is allowed to interact with the real time domain.
  </ul>
  
  \section sec_2 What if..
  \subsection ssec_1 ...I have to deal with multiple targets?
  You must use the framework in full mode as shown in Section \ref sec_1_1.
  \subsection ssec_2 ...I need a small framework to develop only one target? 
  You must use the framework in standalone mode as shown in Section \ref sec_1_2.

*/

/*! \page connect_difference Compare connection between Old server and New (0.6) server
 * 
 * Here we discuss the connection procedure with RTAI-XML server. In particular we compare the connection
 * in the old server (<0.6) and new server (>=0.6).
 * 
 * \section sec_d_1 Connect to the old server (<0.6)
 * The following figure shows the actor involved in the RTAI-XML domain.
 * \image html old_connect.png
 * where there are 2 actors:
 * <ul>
 * 	<li><b>rtaixml</b>: that receives the connection from internet through tConnect XMLRPC command and
 * 						interact with the targets to collect information and dispacth them to the client.
 * 	<li><b>targets</b>: the real time executables that perform the control task.
 * </ul>
 * In order to connect the client to a target to get all the information it is necessary to send a 
 * tConnect XMLRPC command to RTAI-XML server that provide you all the information about your target.
 * After this connection you can interact with the target as usual.
 * 
 * \section sec_d_2 Connect to the new server (>=0.6)
 * The following figure shows the actor involved in the RTAI-XML domain.
 * \image html functional.jpg
 * where there are 3 actors:
 * <ul>
 *  <li><b>rt manager</b>: 	that receive the first connection from internet through ConnectionRequest
 * 							XMLRPC command. Through this you ask for a connection to your target to the
 * 							manager of the real time domain. It parse your request check availablity
 * 							of the resource and provide you the information to connect to a rtaixml
 * 							server.
 * 	<li><b>rtaixml</b>: that receives the connection from internet through tConnect XMLRPC command and
 * 						interact with the targets to collect information and dispacth them to the client.
 * 						In order to successful call tConnect XMLRPC command you must provide the 
 * 						information returned by rt manager (ConnectionRequest).
 * 	<li><b>targets</b>: the real time executables that perform the control task.
 * </ul>
 * In order to connect the client to a target to get all the information it is necessary to send a 
 * ConnectionRequest to rt manager and exploit the returned information to call
 * tConnect XMLRPC command of rtaixml. 
 * After this 2 steps connection you can interact with the target as usual.
 * 
 * 
 */

/*! \page install_procedure How to compile and install it
 * 
 * RTAI-XML from this version on include a install script. In order to install RTAI-XML suite on your RTAI box you must follow this steps:
 * 
 * <ul>
 * 	<li> download the source code from http://artist.dsi.unifi.it/rtaixml (you probably have already done that if you are reading this file! :-) )
 *		<li> uncompress the zip file in a directory
 *		<li> enter the directory and run: <br/>
 *				> make all <br/>
 *				this command compiles all the source code. The compilation process has been tested with gcc version 3.3, but it should be working also with newer version.
 *		<li> as root, run:<br/>
 * 			> make install <br/>
 *				that install RTAI-XML suite in your system. It should work well in most	system and if you want to personalise your installation you have to change the first lines of Makefile that contain information about the installation. They should be quite easy to understand and modify.
 * </ul>
 * Now you are ready to enjoy your fresh installation!
 * The install procedure by default create a directory in /usr/realtime called rtaixml where put all the necessary files plus some symbolic link in  /usr/realtime/bin. In such a directory you will find rtmanager that is the real time manager that should be always running and rtaixml that is the real time server.
 * 
 */
	
/*! \page use How to use it
 * 
 * Quick 3 steps procedure:
 * 
 * <ul>
 * 	<li> configure your target in the script directory as shown in \ref configure;
 *		<li> start rtmanager (forget everything about rtaixml and target execution! rtmanager will start your target as you requested via XML-RPC and execute 	the needed RTAI-XML servers to manage them, but this is completely tranparent to you!)
 *		<li> Execute your client (i.e. jRtaiLab) and connect to the target name as specified in the script file.
 * </ul>
 * 
 */	

/*! \page configure How to configure the target script file
 * 
 * In order to register your target in the RTAI-XML domain you must write your own script file. This should be saved in the script directory (usually /usr/realtime/rtaixml/script) with the name you intend to call your target. i.e. if you want to connect to your target through XML-RPC with the name MYTARGET, MYTARGET must be name of the script (maximum 8 chars). To compile  the script use TEST (located in the script directory) as model.
 *
 * The parameters in the script file are the following:
 * <ul>
 * 	<li> <b>depend</b> this option let you to start another target before this one. It is possible to create in a recursive way several target. This has to be set as the script file name related to the target you want to start. Optional.
 * 	<li> <b>delay</b> introduce a delay before target execution. Useful to execute depending targets. Optional.
 * 	<li> <b>target</b> file name of the target executable, only the filename.
 * 	<li> <b>dir</b> absolute path where the target executable is located
 * 	<li> <b>priority</b> Real time priority of the target in the RTAI domain.
 * 	<li> <b>time</b> Final time after which the target ends.
 * 	<li> <b>wait</b> if true the target starts in wait.
 * 	<li> <b>verbose</b> target verbose output
 * 	<li> <b>rts</b> Name of the real time Scope. You must set this string different from the other target realtime names [taskname(equal to the scrpt name) - scope	- led]
 * 	<li> <b>rte</b> Name of the real time Digital/Led. You must set this string different from the other target realtime names [taskname(equal to the scrpt name) - scope - led]
 * </ul>
 * 
 */



#ifndef RTAIXML_MASTER_H_
#define RTAIXML_MASTER_H_

/** \namespace rtaixml_domain
 * \brief The RTAI-XML server defines an operation domain called RTAI-XML domain. 
 * 
 * The RTAI-XML server defines an operation domain called RTAI-XML domain. 
 * You can find three different objects in this domain related to their position 
 * inside the RTAI-XML domain:
 * \li the realtime targets are situated inside the domain and work in HRT. 
 * They can be generated using RCP techniques based on RTAI-Lab, or implemented 
 * directly using the API of RTAI. In both cases you have to observe the 
 * rigorous RT communication interfaces with the RTAI-XML server.
 * \li the RTAI-XML server is located on the real time domain border. It has 
 * to manage some executing targets, connecting and disconnecting them from the 
 * domain, obtaining the structure and changing, if required, their parameters 
 * in real time. Overall it has to supervise the communication with the RT 
 * processes. On the other side it gives an external interface that allows to 
 * interact with the various targets inside the domain. Such interface is based 
 * on XML over HTTP and provides the possibility to obtain information about the 
 * RTAI-XML domain. Such information are about both the state of the domain and 
 * the state of every target and provide the possibility to interact with these 
 * during their execution.
 * \li the external applications are the users of the RTAI-XML server services. 
 * Such applications are not developed in a real time framework and can be 
 * implemented in every programming languages. They communicate with the RTAI-XML 
 * server using the XML remote procedure calls.
 * 
 * A framework developed according this logic shows the necessary flexibility 
 * to adapt itself to many applications. The flexibility 
 * and the dynamism of the RTAI-XML domain is provided both on the internal side 
 * and the external one.
 * Towards the real time targets, the dynamism is insured both at the programming 
 * level and the interface level. At the programming level the RTAI-XML server 
 * can communicate with the targets that have been manually written or 
 * automatically generated by RCP tools. At the interface level it allows the 
 * definition of a communication protocol between the target and the RTAI-XML 
 * server.
 * On the external side the flexibility and the power of a Web services approach 
 * is provided. Such architecture is completely independent from the hardware 
 * outside the RTAI-XML domain. At the same time it allows to communicate and 
 * interact between generic external applications and the RTAI-XML domain.
 */


//#include "Constants.h"
#include "RTDomain.h"
#include "RTDomainMEM.h"
#include "XmlRpc.h"

#include <string>
#include <sstream>
#include <semaphore.h> 
#include <iostream>
#include <stdio.h>
#include <pthread.h>
#include <sys/poll.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <getopt.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <sys/wait.h>
#include <sys/io.h>
#include <sys/poll.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <exception>

namespace rtaixml_domain {
	
#define RTAIXML_MASTER_VERSION 	"0.2"
//#define RTAILAB_VERSION   		"3.0.5"
#define RTAIXML_VERSION	    	"0.8.2"

//! true debug mode on. It prints on the standard output "useful" information.
bool IS_DEBUG = false;

}

#endif /*RTAIXML_MASTER_H_*/
